package org.zeroconf;

import org.gudy.azureus2.plugins.*;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadManagerListener;
import org.gudy.azureus2.plugins.logging.LoggerChannel;

import com.apple.dnssd.*;

class Advertisement implements RegisterListener {

	public DNSSDRegistration	registration;		// object that maintains our connection advertisement
	static final String zeroconfRegType = "_bittorrent._tcp";
	String ourName;
	int port;
	String infohash;
	LoggerChannel logger;
	
	public Advertisement (Download download, PluginInterface pluginIf, LoggerChannel logger) throws DNSSDException {

			this.logger = logger;
			//So we need the infohash, the port & the userid
			ourName = new String(download.getDownloadPeerId());
			infohash = byteArrayToHexString(download.getTorrent().getHash());
			port = pluginIf.getPluginconfig().getUnsafeIntParameter("TCP.Listen.Port");
			
		logger.log("Registering: Name: " + ourName + " Hash: " + infohash + " Port: " + port); 
		try {
			registration = DNSSD.register( 0, 0, ourName, zeroconfRegType + "," + infohash, "", "", port, null, this);
			//registration = DNSSD.register( 0, 0, ourName, zeroconfRegType, "", "", port, null, this);
		}
		catch (DNSSDException e) {
			logger.log("DNSSD.register for " + ourName + " " + infohash + "failed!", e);
		}
	}

	public void stop() {
		registration.stop();
	}
	
	//RegisterListener callback: Confirm our registration	
	public void	serviceRegistered( DNSSDRegistration registration, int flags, String serviceName, String regType, String domain) {
		ourName = serviceName;		// might have been renamed on collision
		logger.log("Service Registered as: " + ourName );
	}

	//DNSSD Common callback	
	public void	operationFailed( DNSSDService service, int errorCode) {
		logger.log( "Service reported error " + String.valueOf( errorCode));
	}
	
	//Convert byte[] to Hex String
	private String byteArrayToHexString(byte[] data) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < data.length; i++) {
			buffer.append(intToHexChar((data[i]>>>4)&0x0F));
			buffer.append(intToHexChar(data[i]&0x0F));
		}
		return buffer.toString();
	}
	
	private static char intToHexChar(int i) {
        if ((0 <= i) && (i <= 9)) {
            return (char) ('0' + i);
        } 
		else {
            return (char) ('a' + (i - 10));
	}
	
}
	
}
