package org.zeroconf;

import java.net.*;

import com.apple.dnssd.*;

import org.gudy.azureus2.plugins.*;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.peers.PeerManager;
import org.gudy.azureus2.plugins.logging.LoggerChannel;

class Discovery implements ResolveListener, BrowseListener {
	
	static final String zeroconfRegType = "_bittorrent._tcp";
	public DNSSDService		browser;			// object that actively browses for other peers
	public DNSSDService		resolver;			// object that resolves other peers
	
	String ourName;
	int port;
	String infohash;
	Download download;
	LoggerChannel logger;
	PeerAdder adder[];
	int k = 0;

	//Plugin interface which gives access to the Azureus plugin API
	private PluginInterface pluginInterface;
	
	

	public Discovery(Download download, LoggerChannel logger) throws DNSSDException {
			this.download = download;
			this.logger = logger;
			
			//So we need the infohash, the port & the userid
			infohash = byteArrayToHexString(download.getTorrent().getHash());
			try {
				browser = DNSSD.browse( 0, 0, zeroconfRegType + "," + infohash, "", this);
				//browser = DNSSD.browse( 0, 0, zeroconfRegType, "", this);
			}
			catch (DNSSDException e) {
				logger.log("DNSSD.browse for " + infohash + " failed!", e);
			}
	}

	//Browser callback: Resolve services we discover
	public void serviceFound(DNSSDService browser, int flags, int ifIndex, String name, String regType, String domain) {
		//System.out.println("Add -> flags:" + flags + ", ifIndex:" + ifIndex + ", Name:" + name + ", Type:" + regType + "Domain:" + domain);
		try {
			resolver = DNSSD.resolve(flags, ifIndex, name, regType, domain, this);
		}
		catch (DNSSDException e) {
			logger.log("DNSSD.resolve for " + name + "failed!", e);
		}
	}
	
	//Browser callback: Print a line when services go away
	public void serviceLost(DNSSDService browser, int flags, int ifIndex, String name, String regType, String domain) {
		//System.out.println("Rmv -> flags:" + flags + ", ifIndex:" + ifIndex + ", Name:" + name + ", Type:" + regType + "Domain:" + domain);
		// TODO: remove this peer from the list of peers but for now we just let it time out.
	}

	//Resolve callback: 	
	public void	serviceResolved( DNSSDService resolver, int flags, int ifIndex, String fullName, 
												String hostName, int port, TXTRecord txtRecord) {
		// Start a record query to obtain IP address from hostname
		try {
			DNSSD.queryRecord( 0, ifIndex, hostName, 1, 1, new PeerAdder(download, port)); //faster & safer
		}
		catch (DNSSDException e) {
			logger.log("DNSSD.queryRecord for " + hostName + "failed!", e);
		}
		resolver.stop();
		/*
		try {
			System.out.println("Test: getByName returned IP: " + InetAddress.getByName(hostName).getHostAddress());
		}
		catch (UnknownHostException e) {
				System.out.println("Got answer from: Unknown host!");
			return;
		} */
			
	}
	
	public void stop() {
		browser.stop();
	}
	
	//DNSSD Common callback	
	public void	operationFailed( DNSSDService service, int errorCode) {
		logger.log( "Service reported error " + String.valueOf( errorCode));
	}
	
	//Convert byte[] to Hex String
	private String byteArrayToHexString(byte[] data) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < data.length; i++) {
			buffer.append(intToHexChar((data[i]>>>4)&0x0F));
			buffer.append(intToHexChar(data[i]&0x0F));
		}
		return buffer.toString();
	}
	
	private static char intToHexChar(int i) {
        if ((0 <= i) && (i <= 9)) {
            return (char) ('0' + i);
        } 
		else {
            return (char) ('a' + (i - 10));
		}
	}
		
}